﻿using ELTE.BoardGame.Persistence;
using System;
using System.Linq;

namespace ELTE.BoardGame.Model
{
    /// <summary>
    /// Táblás játék modell típusa.
    /// </summary>
    public abstract class BoardGameModel
    {
        #region Protected fields

        protected readonly PlayerData[,] gameBoard; // játéktábla
        protected readonly IPersistence persistence; // adatelérés

        #endregion

        #region Events

        /// <summary>
        /// Játék megnyerésének eseménye.
        /// </summary>
        public event EventHandler<GameWonEventArgs> GameWon;

        /// <summary>
        /// Játék végének eseménye.
        /// </summary>
        public event EventHandler GameOver;

        /// <summary>
        /// Mezőváltozás eseménye.
        /// </summary>
        public event EventHandler<FieldChangedEventArgs> FieldChanged;

        #endregion

        #region Constructors

        /// <summary>
        /// Táblás játék modell példányosítása.
        /// </summary>
        /// <param name="boardSize">Játéktábla vízszintes, illetve függőleges mérete.</param>
        /// <param name="persistence">Perzisztencia szolgátlatás.</param>
        protected BoardGameModel(Int32 boardSize , IPersistence persistence)
        {
            if (boardSize < 1)
                throw new ArgumentOutOfRangeException("boardSize", "The board size is less than 1.");

            this.gameBoard = new PlayerData[boardSize, boardSize]; // mátrix létrehozása
            this.persistence = persistence;

            this.NewGame();
        }

        #endregion

        #region Public properties

        /// <summary>
        /// Aktuális játékos lekérdezése.
        /// </summary>
        public PlayerData CurrentPlayer { get; private set; }

        /// <summary>
        /// Lépésszám lekérdezése.
        /// </summary>
        public Int32 StepNumber { get; private set; }

        /// <summary>
        /// Tábla magasságának lekérdezése.
        /// </summary>
        public Int32 BoardHeight { get { return this.gameBoard.GetLength(1); } }

        /// <summary>
        /// Tábla szélességének lekérdezése.
        /// </summary>
        public Int32 BoardWidth { get { return this.gameBoard.GetLength(0); } }

        /// <summary>
        /// Mezőérték lekérdezése, vagy beállítása.
        /// </summary>
        /// <param name="x">Oszlop index.</param>
        /// <param name="y">Sor index.</param>
        /// <returns>A mező értéke.</returns>
        public PlayerData this[Int32 x, Int32 y]
        {
            get
            {
                if (x < 0 || x >= this.BoardWidth) // ellenőrizzük a tartományt
                    throw new InvalidOperationException("Bad column index.");
                if (y < 0 || y >= this.BoardHeight)
                    throw new InvalidOperationException("Bad row index.");

                return this.gameBoard[x, y];
            }
        }

        #endregion

        #region Public methods

        /// <summary>
        /// Új játék indítása.
        /// </summary>
        public void NewGame()
        {
            for (Int32 i = 0; i < this.BoardWidth; i++)
                for (Int32 j = 0; j < this.BoardHeight; j++)
                {
                    this.gameBoard[i, j] = PlayerData.NoPlayer; // a játékosok pozícióit töröljük
                }

            this.StepNumber = 0;
            this.CurrentPlayer = PlayerData.PlayerX; // először az X lép
        }

        /// <summary>
        /// Játék léptetése.
        /// </summary>
        /// <param name="x">Oszlop index.</param>
        /// <param name="y">Sor index.</param>
        public void StepGame(Int32 x, Int32 y)
        {
            if (x < 0 || x > this.gameBoard.GetLength(0)) // ellenőrizzük a tartományt
                throw new ArgumentOutOfRangeException("x", "Bad column index.");
            if (y < 0 || y > this.gameBoard.GetLength(1))
                throw new ArgumentOutOfRangeException("y", "Bad row index.");
            if (this.StepNumber >= this.BoardWidth * this.BoardHeight) // ellenőrizzük a lépésszámot
                throw new InvalidOperationException("Game is over!");
            if (this.gameBoard[x, y] != PlayerData.NoPlayer) // ellenőrizzük a mezőt
                throw new InvalidOperationException("Field is not empty!");

            this.StepGameInternal(x, y);

            this.StepNumber++;
            this.CurrentPlayer = this.CurrentPlayer == PlayerData.PlayerO ? PlayerData.PlayerX : PlayerData.PlayerO;
            // beállítjuk a következő játékost

            this.CheckGame();
        }

        /// <summary>
        /// Játék betöltése.
        /// </summary>
        /// <param name="path">Elérési útvonal.</param>
        public void LoadGame(String path)
        {
            if (this.persistence == null)
                return;

            // végrehajtjuk a betöltést és a konverziót
            PlayerData[] values = this.persistence.Load(path).Select(MapFieldToPlayer).ToArray();

            if (values.Length != this.gameBoard.Length)
                throw new DataException("Error occured during game loading.");

            if (values.Count(value => value == PlayerData.PlayerO) != values.Count(value => value == PlayerData.PlayerX) && values.Count(value => value == PlayerData.PlayerO) + 1 != values.Count(value => value == PlayerData.PlayerX))
                // ellenőrizzük, hogy megfelelő számú lépés történt-e a játékosok részéről (leszámoljuk, hány lépés történt az egyes játékosok részéről)
                throw new DataException("Error occured during game loading.");

            this.StepNumber = values.Count(value => value != PlayerData.NoPlayer); // leszámoljuk, hány lépés történt
            this.CurrentPlayer = this.StepNumber % 2 == 0 ? PlayerData.PlayerX : PlayerData.PlayerO; // a lépésszámból kijön az aktuális játékos

            // beállítjuk az értékeket
            for (Int32 i = 0; i < this.BoardWidth; i++)
                for (Int32 j = 0; j < this.BoardHeight; j++)
                {
                    this.gameBoard[i, j] = values[i * this.BoardWidth + j];

                    this.OnFieldChanged(i, j, this.gameBoard[i, j]);
                }

            this.CheckGame();
        }

        /// <summary>
        /// Játék mentése.
        /// </summary>
        /// <param name="path">Elérési útvonal.</param>
        public void SaveGame(String path)
        {
            if (this.persistence == null)
                return;

            // az értékeket kimásoljuk egy új tömbbe
            FieldData[] values = new FieldData[this.gameBoard.Length];
            for (Int32 i = 0; i < this.BoardWidth; i++)
                for (Int32 j = 0; j < this.BoardHeight; j++)
                {
                    values[i * this.BoardWidth + j] =  this.MapPlayerToField(this.gameBoard[i, j]); // konvertáljuk az értéket
                }

            // végrehajtjuk a mentést
            this.persistence.Save(path, values);
        }

        #endregion

        #region Protected methods

        /// <summary>
        /// Játék léptetése.
        /// </summary>
        /// <param name="x">Oszlop index.</param>
        /// <param name="y">Sor index.</param>
        protected abstract void StepGameInternal(Int32 x, Int32 y);

        /// <summary>
        /// Játék ellenőrzése.
        /// </summary>
        protected abstract void CheckGame();

        #endregion

        #region Private methods

        /// <summary>
        /// Játékos leképezése játékosra.
        /// </summary>
        /// <param name="field">A mező.</param>
        /// <returns>A megfelelő játékos.</returns>
        private FieldData MapPlayerToField(PlayerData player)
        {
            return (FieldData)(Int32)player;
        }

        /// <summary>
        /// Játékos leképezése játékosra.
        /// </summary>
        /// <param name="field">A mező.</param>
        /// <returns>A megfelelő játékos.</returns>
        private PlayerData MapFieldToPlayer(FieldData field)
        {
            return (PlayerData)(Int32)field;
        }

        #endregion

        #region Event triggers

        /// <summary>
        /// Játék megnyerésének eseménykiváltása.
        /// </summary>
        /// <param name="player">A győztes játékos.</param>
        protected void OnGameWon(PlayerData player)
        {
            if (this.GameWon != null)
                this.GameWon(this, new GameWonEventArgs(player));
        }

        /// <summary>
        /// Játék végének eseménykiváltása.
        /// </summary>
        protected void OnGameOver()
        {
            if (this.GameOver != null)
                this.GameOver(this, EventArgs.Empty);
        }

        /// <summary>
        /// Mezőváltozás eseménykiváltása.
        /// </summary>
        /// <param name="x">Oszlop index.</param>
        /// <param name="y">Sor index.</param>
        /// <param name="player">Játékos.</param>
        protected void OnFieldChanged(Int32 x, Int32 y, PlayerData player)
        {
            if (this.FieldChanged != null)
                this.FieldChanged(this, new FieldChangedEventArgs(x, y, player));
        }

        #endregion
    }
}
